from _typeshed import Incomplete
from typing import Any

class GraphCommands:
    def commit(self): ...
    version: Any
    def query(
        self, q, params: Incomplete | None = ..., timeout: Incomplete | None = ..., read_only: bool = ..., profile: bool = ...
    ): ...
    def merge(self, pattern): ...
    def delete(self): ...
    nodes: Any
    edges: Any
    def flush(self) -> None: ...
    def explain(self, query, params: Incomplete | None = ...): ...
    def bulk(self, **kwargs) -> None: ...
    def profile(self, query): ...
    def slowlog(self): ...
    def config(self, name, value: Incomplete | None = ..., set: bool = ...): ...
    def list_keys(self): ...
