from typing import Any

class Path:
    append_type: Any
    def __init__(self, nodes, edges) -> None: ...
    @classmethod
    def new_empty_path(cls): ...
    def nodes(self): ...
    def edges(self): ...
    def get_node(self, index): ...
    def get_relationship(self, index): ...
    def first_node(self): ...
    def last_node(self): ...
    def edge_count(self): ...
    def nodes_count(self): ...
    def add_node(self, node): ...
    def add_edge(self, edge): ...
    def __eq__(self, other): ...
